@@ -1,4 +1,5 @@
 """Constants for the Multilevel Switch CC."""
+
 from __future__ import annotations
 
 from enum import IntEnum
@@ -13,6 +14,13 @@ COVER_DOWN_PROPERTY = "Down"
 COVER_OFF_PROPERTY = "Off"
 
 
+class CoverStates(IntEnum):
+    """Enum with all (known/used) Z-Wave Cover States."""
+
+    CLOSED = 0
+    OPEN = 99
+
+
 class MultilevelSwitchCommand(IntEnum):
     """Enum for known multilevel switch notifications."""
 
