@@ -1,23 +1,23 @@
 """Provide a model for the Z-Wave JS controller."""
+
 from __future__ import annotations
 
 from dataclasses import dataclass
 from typing import TYPE_CHECKING, Any, Literal, cast
 
-from zwave_js_server.model.node.firmware import (
-    NodeFirmwareUpdateFileInfo,
-    NodeFirmwareUpdateInfo,
-)
+from zwave_js_server.model.node.firmware import NodeFirmwareUpdateInfo
 
 from ...const import (
     MINIMUM_QR_STRING_LENGTH,
+    AssociationCheckResult,
+    ControllerStatus,
     ExclusionStrategy,
     InclusionState,
     InclusionStrategy,
     NodeType,
     QRCodeVersion,
-    RemoveNodeReason,
     RFRegion,
+    RemoveNodeReason,
     ZwaveFeature,
 )
 from ...event import Event, EventBase
@@ -33,6 +33,11 @@ from .inclusion_and_provisioning import (
     ProvisioningEntry,
     QRProvisioningInformation,
 )
+from .rebuild_routes import (
+    RebuildRoutesOptions,
+    RebuildRoutesOptionsDataType,
+    RebuildRoutesStatus,
+)
 from .statistics import (
     ControllerLifelineRoutes,
     ControllerStatistics,
@@ -67,12 +72,13 @@ class NVMProgress:
 class Controller(EventBase):
     """Represent a Z-Wave JS controller."""
 
-    def __init__(self, client: "Client", state: dict) -> None:
+    def __init__(self, client: Client, state: dict) -> None:
         """Initialize controller."""
         super().__init__()
         self.client = client
         self.nodes: dict[int, Node] = {}
-        self._heal_network_progress: dict[int, str] | None = None
+        self._rebuild_routes_progress: dict[Node, RebuildRoutesStatus] | None = None
+        self._last_rebuild_routes_result: dict[Node, RebuildRoutesStatus] | None = None
         self._statistics = ControllerStatistics(DEFAULT_CONTROLLER_STATISTICS)
         self._firmware_update_progress: ControllerFirmwareUpdateProgress | None = None
         for node_state in state["nodes"]:
@@ -94,6 +100,15 @@ class Controller(EventBase):
             return False
         return self.home_id == other.home_id
 
+    def _generate_rebuild_routes_status(
+        self, data: dict[str, str]
+    ) -> dict[Node, RebuildRoutesStatus]:
+        """Generate rebuild routes status."""
+        return {
+            self.nodes[int(node_id)]: RebuildRoutesStatus(status)
+            for node_id, status in data.items()
+        }
+
     @property
     def sdk_version(self) -> str | None:
         """Return sdk_version."""
@@ -189,9 +204,9 @@ class Controller(EventBase):
         return self.data.get("supportsTimers")
 
     @property
-    def is_heal_network_active(self) -> bool | None:
-        """Return is_heal_network_active."""
-        return self.data.get("isHealNetworkActive")
+    def is_rebuilding_routes(self) -> bool | None:
+        """Return is_rebuilding_routes."""
+        return self.data.get("isRebuildingRoutes")
 
     @property
     def statistics(self) -> ControllerStatistics:
@@ -199,9 +214,14 @@ class Controller(EventBase):
         return self._statistics
 
     @property
-    def heal_network_progress(self) -> dict[int, str] | None:
-        """Return heal network progress state."""
-        return self._heal_network_progress
+    def rebuild_routes_progress(self) -> dict[Node, RebuildRoutesStatus] | None:
+        """Return rebuild routes progress state."""
+        return self._rebuild_routes_progress
+
+    @property
+    def last_rebuild_routes_result(self) -> dict[Node, RebuildRoutesStatus] | None:
+        """Return the last rebuild routes result."""
+        return self._last_rebuild_routes_result
 
     @property
     def inclusion_state(self) -> InclusionState:
@@ -220,12 +240,26 @@ class Controller(EventBase):
         """Return firmware update progress."""
         return self._firmware_update_progress
 
+    @property
+    def status(self) -> ControllerStatus:
+        """Return status."""
+        return ControllerStatus(self.data["status"])
+
+    @property
+    def supports_long_range(self) -> bool | None:
+        """Return whether controller supports long range or not."""
+        return self.data.get("supportsLongRange")
+
     def update(self, data: ControllerDataType) -> None:
         """Update controller data."""
         self.data = data
         self._statistics = ControllerStatistics(
             self.data.get("statistics", DEFAULT_CONTROLLER_STATISTICS)
         )
+        if "rebuildRoutesProgress" in self.data:
+            self._rebuild_routes_progress = self._generate_rebuild_routes_status(
+                self.data["rebuildRoutesProgress"]
+            )
 
     async def async_begin_inclusion(
         self,
@@ -247,7 +281,8 @@ class Controller(EventBase):
         if force_security is not None:
             if inclusion_strategy != InclusionStrategy.DEFAULT:
                 raise ValueError(
-                    "`forceSecurity` option is only supported with inclusion_strategy=DEFAULT"
+                    "`forceSecurity` option is only supported with inclusion_strategy="
+                    "DEFAULT"
                 )
             options["forceSecurity"] = force_security
 
@@ -256,7 +291,8 @@ class Controller(EventBase):
         if provisioning is not None:
             if inclusion_strategy != InclusionStrategy.SECURITY_S2:
                 raise ValueError(
-                    "`provisioning` option is only supported with inclusion_strategy=SECURITY_S2"
+                    "`provisioning` option is only supported with inclusion_strategy="
+                    "SECURITY_S2"
                 )
 
             if dsk is not None:
@@ -269,8 +305,8 @@ class Controller(EventBase):
                     provisioning
                 ) < MINIMUM_QR_STRING_LENGTH or not provisioning.startswith("90"):
                     raise ValueError(
-                        f"QR code string must be at least {MINIMUM_QR_STRING_LENGTH} characters "
-                        "long and start with `90`"
+                        f"QR code string must be at least {MINIMUM_QR_STRING_LENGTH} "
+                        "characters long and start with `90`"
                     )
                 options["provisioning"] = provisioning
             # If we get a Smart Start QR code, we provision the node and return because
@@ -280,8 +316,8 @@ class Controller(EventBase):
                 and provisioning.version == QRCodeVersion.SMART_START
             ):
                 raise ValueError(
-                    "Smart Start QR codes can't use the normal inclusion process. Use the "
-                    "provision_smart_start_node command to provision this device."
+                    "Smart Start QR codes can't use the normal inclusion process. Use "
+                    "the provision_smart_start_node command to provision this device."
                 )
             # Otherwise we assume the data is ProvisioningEntry or
             # QRProvisioningInformation that is not a Smart Start QR code
@@ -321,9 +357,11 @@ class Controller(EventBase):
         await self.client.async_send_command(
             {
                 "command": "controller.provision_smart_start_node",
-                "entry": provisioning_info
-                if isinstance(provisioning_info, str)
-                else provisioning_info.to_dict(),
+                "entry": (
+                    provisioning_info
+                    if isinstance(provisioning_info, str)
+                    else provisioning_info.to_dict()
+                ),
             },
             require_schema=11,
         )
@@ -372,6 +410,12 @@ class Controller(EventBase):
         )
         return cast(bool, data["success"])
 
+    async def async_cancel_secure_bootstrap_s2(self) -> None:
+        """Send cancelSecureBootstrapS2 command to Controller."""
+        await self.client.async_send_command(
+            {"command": "controller.cancel_secure_bootstrap_s2"}, require_schema=40
+        )
+
     async def async_begin_exclusion(
         self, strategy: ExclusionStrategy | None = None
     ) -> bool:
@@ -417,7 +461,8 @@ class Controller(EventBase):
         if force_security is not None:
             if inclusion_strategy != InclusionStrategy.DEFAULT:
                 raise ValueError(
-                    "`forceSecurity` option is only supported with inclusion_strategy=DEFAULT"
+                    "`forceSecurity` option is only supported with inclusion_strategy="
+                    "DEFAULT"
                 )
             options["forceSecurity"] = force_security
 
@@ -426,7 +471,8 @@ class Controller(EventBase):
         if provisioning is not None:
             if inclusion_strategy != InclusionStrategy.SECURITY_S2:
                 raise ValueError(
-                    "`provisioning` option is only supported with inclusion_strategy=SECURITY_S2"
+                    "`provisioning` option is only supported with inclusion_strategy="
+                    "SECURITY_S2"
                 )
             # Provisioning option was introduced in Schema 11
             require_schema = 11
@@ -436,8 +482,8 @@ class Controller(EventBase):
                     provisioning
                 ) < MINIMUM_QR_STRING_LENGTH or not provisioning.startswith("90"):
                     raise ValueError(
-                        f"QR code string must be at least {MINIMUM_QR_STRING_LENGTH} characters "
-                        "long and start with `90`"
+                        f"QR code string must be at least {MINIMUM_QR_STRING_LENGTH} "
+                        "characters long and start with `90`"
                     )
                 options["provisioning"] = provisioning
             # Otherwise we assume the data is ProvisioningEntry or
@@ -455,29 +501,35 @@ class Controller(EventBase):
         )
         return cast(bool, data["success"])
 
-    async def async_heal_node(self, node: Node) -> bool:
-        """Send healNode command to Controller."""
+    async def async_rebuild_node_routes(self, node: Node) -> bool:
+        """Send rebuildNodeRoutes command to Controller."""
         data = await self.client.async_send_command(
-            {"command": "controller.heal_node", "nodeId": node.node_id}
+            {"command": "controller.rebuild_node_routes", "nodeId": node.node_id},
+            require_schema=32,
         )
         return cast(bool, data["success"])
 
-    async def async_begin_healing_network(self) -> bool:
-        """Send beginHealingNetwork command to Controller."""
-        data = await self.client.async_send_command(
-            {"command": "controller.begin_healing_network"}
-        )
+    async def async_begin_rebuilding_routes(
+        self, options: RebuildRoutesOptions | None = None
+    ) -> bool:
+        """Send beginRebuildingRoutes command to Controller."""
+        msg: dict[str, str | RebuildRoutesOptionsDataType] = {
+            "command": "controller.begin_rebuilding_routes"
+        }
+        if options:
+            msg["options"] = options.to_dict()
+        data = await self.client.async_send_command(msg, require_schema=32)
         return cast(bool, data["success"])
 
-    async def async_stop_healing_network(self) -> bool:
-        """Send stopHealingNetwork command to Controller."""
+    async def async_stop_rebuilding_routes(self) -> bool:
+        """Send stopRebuildingRoutes command to Controller."""
         data = await self.client.async_send_command(
-            {"command": "controller.stop_healing_network"}
+            {"command": "controller.stop_rebuilding_routes"}, require_schema=32
         )
         success = cast(bool, data["success"])
         if success:
-            self._heal_network_progress = None
-            self.data["isHealNetworkActive"] = False
+            self._rebuild_routes_progress = None
+            self.data["isRebuildingRoutes"] = False
         return success
 
     async def async_is_failed_node(self, node: Node) -> bool:
@@ -529,6 +581,7 @@ class Controller(EventBase):
         for key, association_addresses in data["associations"].items():
             associations[int(key)] = [
                 AssociationAddress(
+                    self,
                     node_id=association_address["nodeId"],
                     endpoint=association_address.get("endpoint"),
                 )
@@ -536,10 +589,10 @@ class Controller(EventBase):
             ]
         return associations
 
-    async def async_is_association_allowed(
+    async def async_check_association(
         self, source: AssociationAddress, group: int, association: AssociationAddress
-    ) -> bool:
-        """Send isAssociationAllowed command to Controller."""
+    ) -> AssociationCheckResult:
+        """Send checkAssociation command to Controller."""
         source_data = {"nodeId": source.node_id}
         if source.endpoint is not None:
             source_data["endpoint"] = source.endpoint
@@ -549,13 +602,14 @@ class Controller(EventBase):
             association_data["endpoint"] = association.endpoint
         data = await self.client.async_send_command(
             {
-                "command": "controller.is_association_allowed",
+                "command": "controller.check_association",
                 **source_data,
                 "group": group,
                 "association": association_data,
-            }
+            },
+            require_schema=37,
         )
-        return cast(bool, data["allowed"])
+        return AssociationCheckResult(data["result"])
 
     async def async_add_associations(
         self,
@@ -750,7 +804,9 @@ class Controller(EventBase):
         )
 
         return {
-            self.nodes[node_id]: ControllerLifelineRoutes(self.client, lifeline_routes)
+            self.nodes[int(node_id)]: ControllerLifelineRoutes(
+                self.client, lifeline_routes
+            )
             for node_id, lifeline_routes in data["routes"].items()
         }
 
@@ -768,7 +824,7 @@ class Controller(EventBase):
         return cast(bool, data["progress"])
 
     async def async_get_available_firmware_updates(
-        self, node: Node, api_key: str, include_prereleases: bool = False
+        self, node: Node, api_key: str, include_prereleases: bool = True
     ) -> list[NodeFirmwareUpdateInfo]:
         """Send getAvailableFirmwareUpdates command to Controller."""
         data = await self.client.async_send_command(
@@ -778,22 +834,22 @@ class Controller(EventBase):
                 "apiKey": api_key,
                 "includePrereleases": include_prereleases,
             },
-            require_schema=24,
+            require_schema=32,
         )
         assert data
         return [NodeFirmwareUpdateInfo.from_dict(update) for update in data["updates"]]
 
     async def async_firmware_update_ota(
-        self, node: Node, updates: list[NodeFirmwareUpdateFileInfo]
+        self, node: Node, update_info: NodeFirmwareUpdateInfo
     ) -> NodeFirmwareUpdateResult:
         """Send firmwareUpdateOTA command to Controller."""
         data = await self.client.async_send_command(
             {
                 "command": "controller.firmware_update_ota",
                 "nodeId": node.node_id,
-                "updates": [update.to_dict() for update in updates],
+                "updateInfo": update_info.to_dict(),
             },
-            require_schema=29,
+            require_schema=32,
         )
         return NodeFirmwareUpdateResult(node, data["result"])
 
@@ -818,10 +874,11 @@ class Controller(EventBase):
         if event.data["source"] != "controller":
             # TODO decide what to do here
             print(
-                f"Controller doesn't know how to handle/forward this event: {event.data}"
+                "Controller doesn't know how to handle/forward this event: "
+                f"{event.data}"
             )
 
-        CONTROLLER_EVENT_MODEL_MAP[event.type](**event.data)
+        CONTROLLER_EVENT_MODEL_MAP[event.type].from_dict(event.data)
 
         self._handle_event_protocol(event)
 
@@ -830,9 +887,9 @@ class Controller(EventBase):
 
     def handle_firmware_update_progress(self, event: Event) -> None:
         """Process a firmware update progress event."""
-        self._firmware_update_progress = event.data[
-            "firmware_update_progress"
-        ] = ControllerFirmwareUpdateProgress(event.data["progress"])
+        self._firmware_update_progress = event.data["firmware_update_progress"] = (
+            ControllerFirmwareUpdateProgress(event.data["progress"])
+        )
 
     def handle_firmware_update_finished(self, event: Event) -> None:
         """Process a firmware update finished event."""
@@ -850,6 +907,10 @@ class Controller(EventBase):
     def handle_inclusion_started(self, event: Event) -> None:
         """Process an inclusion started event."""
 
+    def handle_inclusion_state_changed(self, event: Event) -> None:
+        """Process an inclusion state changed event."""
+        self.data["inclusionState"] = event.data["state"]
+
     def handle_exclusion_started(self, event: Event) -> None:
         """Process an exclusion started event."""
 
@@ -874,16 +935,20 @@ class Controller(EventBase):
         # Remove client from node since it's no longer connected to the controller
         event.data["node"].client = None
 
-    def handle_heal_network_progress(self, event: Event) -> None:
-        """Process a heal network progress event."""
-        self._heal_network_progress = event.data["progress"].copy()
-        self.data["isHealNetworkActive"] = True
+    def handle_rebuild_routes_progress(self, event: Event) -> None:
+        """Process a rebuild routes progress event."""
+        self._rebuild_routes_progress = self._generate_rebuild_routes_status(
+            event.data["progress"]
+        )
+        self.data["isRebuildingRoutes"] = True
 
-    def handle_heal_network_done(self, event: Event) -> None:
-        """Process a heal network done event."""
-        # pylint: disable=unused-argument
-        self._heal_network_progress = None
-        self.data["isHealNetworkActive"] = False
+    def handle_rebuild_routes_done(self, event: Event) -> None:
+        """Process a rebuild routes done event."""
+        self._last_rebuild_routes_result = self._generate_rebuild_routes_status(
+            event.data["result"]
+        )
+        self._rebuild_routes_progress = None
+        self.data["isRebuildingRoutes"] = False
 
     def handle_statistics_updated(self, event: Event) -> None:
         """Process a statistics updated event."""
@@ -921,3 +986,14 @@ class Controller(EventBase):
         event.data["nvm_restore_progress"] = NVMProgress(
             event.data["bytesWritten"], event.data["total"]
         )
+
+    def handle_identify(self, event: Event) -> None:
+        """Process an identify event."""
+        # TODO handle event for unknown node
+        if node := self.nodes.get(event.data["nodeId"]):
+            event.data["node"] = node
+
+    def handle_status_changed(self, event: Event) -> None:
+        """Process a status changed event."""
+        self.data["status"] = event.data["status"]
+        event.data["status"] = ControllerStatus(event.data["status"])
