@@ -3,6 +3,7 @@ Model for a Zwave Node's device config.
 
 https://zwave-js.github.io/node-zwave-js/#/api/node?id=deviceconfig
 """
+
 from __future__ import annotations
 
 from typing import Any, Literal, TypedDict
@@ -164,7 +165,7 @@ class DeviceConfig:
 
     @property
     def manufacturer_id(self) -> str | None:  # TODO: In the dump this is an int.
-        """Return manufacturer id (as defined in the specs) as a 4-digit hexadecimal string."""
+        """Return manufacturer id (as defined in specs) as a 4-digit hex string."""
         return self.data.get("manufacturerId")
 
     @property
